@@ -10,7 +10,7 @@ use Text::Table;
 use Exporter qw( import );
 our @EXPORT = qw( Tabulate );
 
-our $VERSION = 0.007;
+our $VERSION = 0.008;
 
 our %seen;
 
@@ -26,21 +26,25 @@ sub Tabulate ($) {
 
 sub _tblize {
     my ($thing, $run) = @_;
-    return '(* undef *)' unless defined $thing;
+    return 'undef()' unless defined $thing;
     my $r = reftype($thing);
     my $addr = lc sprintf('%x', refaddr($thing));
+    my $circular;
+    if ($r and defined($seen{ $run }->{ $addr })) {
+        $circular++;
+    }
     my $alias
-        = defined $thing
-            ? ($r ? ($r . '(0x' . $addr . ')') : '( scalar )')
-            : '(* undef *)'
-            ;
+        = $r
+        ? $r . '(' . ($seen{ $run }->{ $addr } //= scalar keys %{$seen{ $run }}) . ')'
+        : '( scalar )'
+        ;
     if (my $b = blessed($thing)) {
         $alias = $b . '=' . $alias unless $b eq 'Regexp';
     }
-    if ($alias ne '( scalar )' and $seen{ $run }->{ $alias }++) {
-        return 'CIRCULAR->' . $alias;
+    if ($circular) {
+        return '-> ' . $alias;
     }
-    my $container = Text::Table->new(($alias)x!($alias eq '( scalar )'));
+    my $container = Text::Table->new(($alias)x($alias ne '( scalar )'));
     my $inner = $thing;
     my $snidge = '+';
     if ($r eq 'ARRAY') {
@@ -55,15 +59,18 @@ sub _tblize {
             } : undef
         } @$thing;
         if (@v == @$thing) {
-            my @head = map { \' | ', $_ } sort keys %header;
+            $alias =~ s/ARRAY/ARRAY<HASH>/;
+            $container = Text::Table->new($alias);
+            my @cols = sort keys %header;
+            my @head = map { \' | ', defined($_) ? q{'} . quotemeta($_) . q{'} : 'undef()' } @cols;
             shift @head;
             unshift @head, \' ';
             push @head, \' ';
             $inner = Text::Table->new(@head);
             for my $row (@$thing) {
                 my @body;
-                for my $k (grep { !ref $_ } @head) {
-                    push @body, (exists($row->{ $k }) ? _tblize($row->{ $k }, $run) : '(* no data *)');
+                for my $k (@cols) {
+                    push @body, (exists($row->{ $k }) ? _tblize($row->{ $k }, $run) : '-');
                 }
                 $inner->add(@body);
             }
@@ -83,7 +90,7 @@ sub _tblize {
         my @keys = sort keys %$thing;
         $inner = Text::Table->new();
         for my $k (@keys) {
-            $inner->add($k, '=>', _tblize($thing->{ $k }, $run));
+            $inner->add((defined($k) ? q{'} . quotemeta($k) . q{'} : 'undef()'), '=>', _tblize($thing->{ $k }, $run));
             $snidge = '-';
         }
     }
@@ -91,10 +98,10 @@ sub _tblize {
         $inner = 'sub DUMMY { }'; # TODO for now
     }
     elsif (uc $r eq 'REGEXP') {
-        return "qr/$thing/";
+        return "/$thing/";
     }
     elsif ($r) {
-        $inner = 'REF->' . _tblize($$thing, $run); # TODO for now
+        $inner = _tblize("\\do { " . $$thing . "}", $run); # TODO for now
     }
     else {
         $inner = "'" . quotemeta($inner) . "'";
@@ -117,37 +124,42 @@ Data::Dumper::Table - A more tabular way to Dumper your Data
 
 =head1 VERSION
 
-Version 0.007
+Version 0.008
 
 =head1 SYNOPSIS
 
     use Data::Dumper::Table;
 
+    my $x = [qw(one two three)];
+
     say Tabulate [
-        { foo => 1, bar => 2 },
+        { foo => $x, bar => 2 },
         { foo => 3, bar => { apple => q(or'ange) } },
+        $x,
         [
             { bar => q(baz), flibble => q(quux), flobble => undef() },
             { bar => q(baz2), flobble => qr/foo/ }
         ]
     ];
 
-    ARRAY(0x145dc30) [0] HASH(0x143e148)
-                         ----------
-                         bar => '2'
-                         foo => '1'
-                     [1] HASH(0x14531d8)
-                         --------------------------
-                         bar => HASH(0x1453130)
-                                -------------------
-                                apple => 'or\'ange'
-                         foo => '3'
-                     [2] ARRAY(0x145dba0)
-                          bar    | flibble       | flobble
-                         --------+---------------+---------------
-                          'baz'  | 'quux'        | (* undef *)
-                          'baz2' | (* no data *) | qr/(?^u:foo)/
-
+ARRAY(1) [0] HASH(2)
+             -----------------------------
+             'bar' => '2'
+             'foo' => ARRAY(3) [0] 'one'
+                               [1] 'two'
+                               [2] 'three'
+         [1] HASH(4)
+             ------------------------------
+             'bar' => HASH(5)
+                      ---------------------
+                      'apple' => 'or\'ange'
+             'foo' => '3'
+         [2] -> ARRAY(3)
+         [3] ARRAY<HASH>(6)
+              'bar'  | 'flibble' | 'flobble'
+             --------+-----------+-------------
+              'baz'  | 'quux'    | undef()
+              'baz2' | -         | /(?^u:foo)/
 
 =head1 DESCRIPTION
 
@@ -155,15 +167,19 @@ The goal of Data::Dumper::Table is to provide a more-tabular alternative to Data
 
 =head1 EXPORTED FUNCTIONS
 
-=head2 Tabulate DATA
+=over
 
-Turn the provided DATA into a (hopefully) nicely-formatted table. More verbose and space-hungry than Data::Dumer, but possibly easier to envision.
+=item Tabulate DATA
+
+Turn the provided DATA into a (hopefully) nicely-formatted table. More verbose and space-hungry than Data::Dumper, but possibly easier to read.
+
+=back
 
 =head1 CAVEATS
 
 =head2 This is Alpha software
 
-This module is explicitly alpha-quality software. If you successfully use it in production, you're a braver being than I am.
+This module is explicitly alpha-quality software. If you successfully use it in production, you're a braver being than I am. See also the TODO list for things that are known not to be handled well.
 
 =head1 TODO
 
@@ -175,5 +191,21 @@ Replicate $Data::Dumper::Sortkeys
 
 Replicate $Data::Dumper::Deparse
 
+=head2 Filehandles and other weird globs
+
+Handle them, somehow
+
+=head2 Tied data
+
+Handle it, somehow. Be especially wary of stuff tied to databases, files, and other not-supposed-to-be-dumped things
+
+=head2 Data::Dumper::Table::HTML
+
+Should be a SMOP once this thing's in the air properly ;-)
+
+=head1 LICENSE
+
+Artistic 2.0
+
 =cut
 
